@@ -20,12 +20,12 @@ G_BEGIN_DECLS
 
 #define XRD_TYPE_OVERLAY_DESKTOP_CURSOR xrd_overlay_desktop_cursor_get_type()
 G_DECLARE_FINAL_TYPE (XrdOverlayDesktopCursor, xrd_overlay_desktop_cursor, XRD,
-                      OVERLAY_DESKTOP_CURSOR, OpenVROverlay)
+                      OVERLAY_DESKTOP_CURSOR, GObject)
 
 struct _XrdOverlayDesktopCursor;
 
 XrdOverlayDesktopCursor *
-xrd_overlay_desktop_cursor_new (void);
+xrd_overlay_desktop_cursor_new (GxrContext *context);
 
 G_END_DECLS
 
