@@ -21,10 +21,11 @@ G_BEGIN_DECLS
 
 #define XRD_TYPE_OVERLAY_POINTER_TIP xrd_overlay_pointer_tip_get_type()
 G_DECLARE_FINAL_TYPE (XrdOverlayPointerTip, xrd_overlay_pointer_tip, XRD,
-                      OVERLAY_POINTER_TIP, OpenVROverlay)
+                      OVERLAY_POINTER_TIP, GObject)
 
 XrdOverlayPointerTip *
-xrd_overlay_pointer_tip_new (guint64 controller_index, GulkanClient *gc);
+xrd_overlay_pointer_tip_new (GxrContext    *context,
+                             GxrController *controller);
 
 G_END_DECLS
 
