@@ -13,17 +13,15 @@
 #endif
 
 #include <glib-object.h>
-#include "xrd-overlay-model.h"
-
 #include <gxr.h>
 
 G_BEGIN_DECLS
 
 #define XRD_TYPE_OVERLAY_POINTER xrd_overlay_pointer_get_type()
 G_DECLARE_FINAL_TYPE (XrdOverlayPointer, xrd_overlay_pointer, XRD,
-                      OVERLAY_POINTER, XrdOverlayModel)
+                      OVERLAY_POINTER, GObject)
 
-XrdOverlayPointer *xrd_overlay_pointer_new (guint64 controller_index);
+XrdOverlayPointer *xrd_overlay_pointer_new (void);
 
 G_END_DECLS
 
