@@ -23,7 +23,10 @@ G_DECLARE_FINAL_TYPE (XrdSceneDesktopCursor, xrd_scene_desktop_cursor,
                       XRD, SCENE_DESKTOP_CURSOR, XrdSceneWindow)
 
 
-XrdSceneDesktopCursor *xrd_scene_desktop_cursor_new (void);
+XrdSceneDesktopCursor *
+xrd_scene_desktop_cursor_new (GulkanClient          *gulkan,
+                              VkDescriptorSetLayout *layout,
+                              VkBuffer               lights);
 
 G_END_DECLS
 
