@@ -2,6 +2,7 @@
  * xrdesktop
  * Copyright 2019 Collabora Ltd.
  * Author: Lubosz Sarnecki <lubosz.sarnecki@collabora.com>
+ * Author: Christoph Haag <christoph.haag@collabora.com>
  * SPDX-License-Identifier: MIT
  */
 
@@ -12,12 +13,33 @@
 #include <gulkan.h>
 #include <gxr.h>
 
-#include "xrd-controller.h"
-#include "xrd-scene-pointer.h"
+#include "gxr-controller.h"
 #include "xrd-scene-pointer-tip.h"
+#include "xrd-scene-object.h"
 
 #include "graphene-ext.h"
 
+#if defined(RENDERDOC)
+#include <dlfcn.h>
+#include "renderdoc_app.h"
+static RENDERDOC_API_1_1_2 *rdoc_api = NULL;
+
+static void _init_renderdoc ()
+{
+  if (rdoc_api != NULL)
+    return;
+
+  void *mod = dlopen ("librenderdoc.so", RTLD_NOW | RTLD_NOLOAD);
+  if (mod)
+    {
+      pRENDERDOC_GetAPI RENDERDOC_GetAPI = (pRENDERDOC_GetAPI)dlsym(mod, "RENDERDOC_GetAPI");
+      int ret = RENDERDOC_GetAPI(eRENDERDOC_API_Version_1_1_2, (void **)&rdoc_api);
+      if (ret != 1)
+        g_debug ("Failed to init renderdoc\n");
+    }
+}
+#endif
+
 typedef struct {
   graphene_point3d_t position;
   graphene_point_t   uv;
@@ -36,10 +58,10 @@ typedef struct {
 
 struct _XrdSceneRenderer
 {
-  GulkanClient parent;
+  GulkanRenderer parent;
 
-  VkSampleCountFlagBits msaa_sample_count;
-  float super_sample_scale;
+  VkSampleCountFlagBits sample_count;
+  float render_scale;
 
   VkShaderModule shader_modules[PIPELINE_COUNT * 2];
   VkPipeline pipelines[PIPELINE_COUNT];
@@ -47,16 +69,15 @@ struct _XrdSceneRenderer
   VkPipelineLayout pipeline_layout;
   VkPipelineCache pipeline_cache;
 
-  GulkanFrameBuffer *framebuffer[2];
-
-  uint32_t render_width;
-  uint32_t render_height;
+  GulkanRenderPass *render_pass;
 
   gpointer scene_client;
 
   XrdSceneLights lights;
   GulkanUniformBuffer *lights_buffer;
 
+  GxrContext *context;
+
   void
   (*render_eye) (uint32_t         eye,
                  VkCommandBuffer  cmd_buffer,
@@ -67,7 +88,7 @@ struct _XrdSceneRenderer
   void (*update_lights) (gpointer data);
 };
 
-G_DEFINE_TYPE (XrdSceneRenderer, xrd_scene_renderer, GULKAN_TYPE_CLIENT)
+G_DEFINE_TYPE (XrdSceneRenderer, xrd_scene_renderer, GULKAN_TYPE_RENDERER)
 
 static void
 xrd_scene_renderer_finalize (GObject *gobject);
@@ -82,11 +103,9 @@ xrd_scene_renderer_class_init (XrdSceneRendererClass *klass)
 static void
 xrd_scene_renderer_init (XrdSceneRenderer *self)
 {
-  self->msaa_sample_count = VK_SAMPLE_COUNT_4_BIT;
-  self->super_sample_scale = 1.0f;
+  self->render_scale = 1.0f;
   self->render_eye = NULL;
   self->scene_client = NULL;
-  self->lights_buffer = gulkan_uniform_buffer_new ();
 
   self->lights.active_lights = 0;
   graphene_vec4_t position;
@@ -106,8 +125,7 @@ xrd_scene_renderer_init (XrdSceneRenderer *self)
   self->pipeline_layout = VK_NULL_HANDLE;
   self->pipeline_cache = VK_NULL_HANDLE;
 
-  for (uint32_t eye = 0; eye < 2; eye++)
-    self->framebuffer[eye] = gulkan_frame_buffer_new();
+  self->context = NULL;
 }
 
 static void
@@ -115,16 +133,15 @@ xrd_scene_renderer_finalize (GObject *gobject)
 {
   XrdSceneRenderer *self = XRD_SCENE_RENDERER (gobject);
 
-  VkDevice device = gulkan_client_get_device_handle (GULKAN_CLIENT (self));
+  GulkanClient *gc = gxr_context_get_gulkan (self->context);
+
+  VkDevice device = gulkan_client_get_device_handle (gc);
   if (device != VK_NULL_HANDLE)
     vkDeviceWaitIdle (device);
 
   if (device != VK_NULL_HANDLE)
     {
-      g_object_unref (self->lights_buffer);
-
-      for (uint32_t eye = 0; eye < 2; eye++)
-        g_object_unref (self->framebuffer[eye]);
+      g_clear_object (&self->lights_buffer);
 
       vkDestroyPipelineLayout (device, self->pipeline_layout, NULL);
       vkDestroyDescriptorSetLayout (device, self->descriptor_set_layout, NULL);
@@ -135,61 +152,41 @@ xrd_scene_renderer_finalize (GObject *gobject)
         vkDestroyShaderModule (device, self->shader_modules[i], NULL);
 
       vkDestroyPipelineCache (device, self->pipeline_cache, NULL);
+
+      if (self->render_pass)
+        g_clear_object (&self->render_pass);
     }
 
+  g_clear_object (&self->context);
+
   G_OBJECT_CLASS (xrd_scene_renderer_parent_class)->finalize (gobject);
 }
 
-static XrdSceneRenderer *singleton = NULL;
-
-XrdSceneRenderer *xrd_scene_renderer_get_instance (void)
+XrdSceneRenderer *
+xrd_scene_renderer_new (void)
 {
-  if (singleton == NULL)
-    singleton = (XrdSceneRenderer*) g_object_new (XRD_TYPE_SCENE_RENDERER, 0);
-
-  return singleton;
+  return (XrdSceneRenderer*) g_object_new (XRD_TYPE_SCENE_RENDERER, 0);
 }
 
-void xrd_scene_renderer_destroy_instance (void)
-{
-  g_clear_object (&singleton);
-}
-
-static bool
+static gboolean
 _init_framebuffers (XrdSceneRenderer *self)
 {
-  OpenVRContext *context = openvr_context_get_instance ();
+  VkExtent2D extent;
+  gxr_context_get_render_dimensions (self->context, &extent);
 
-  if (openvr_context_is_valid (context))
-    {
-      openvr_system_get_render_target_size (&self->render_width,
-                                            &self->render_height);
-    }
-  else
-    {
-      g_warning ("Using default render target dimensions.\n");
-      self->render_width = 1080;
-      self->render_height = 1080;
-    }
+  extent.width = (uint32_t) (self->render_scale * (float) extent.width);
+  extent.height = (uint32_t) (self->render_scale * (float) extent.height);
 
-  self->render_width =
-      (uint32_t) (self->super_sample_scale * (float) self->render_width);
-  self->render_height =
-      (uint32_t) (self->super_sample_scale * (float) self->render_height);
+  gulkan_renderer_set_extent (GULKAN_RENDERER (self), extent);
 
-  for (uint32_t eye = 0; eye < 2; eye++)
-    if (!gulkan_frame_buffer_initialize (self->framebuffer[eye],
-                                         gulkan_client_get_device (
-                                           GULKAN_CLIENT (self)),
-                                         self->render_width,
-                                         self->render_height,
-                                         self->msaa_sample_count,
-                                         VK_FORMAT_R8G8B8A8_UNORM))
-      return false;
-  return true;
+  if (!gxr_context_init_framebuffers (self->context, extent, self->sample_count,
+                                      &self->render_pass))
+    return FALSE;
+
+  return TRUE;
 }
 
-static bool
+static gboolean
 _init_shaders (XrdSceneRenderer *self)
 {
   const char *shader_names[PIPELINE_COUNT] = {
@@ -203,16 +200,16 @@ _init_shaders (XrdSceneRenderer *self)
         char path[1024];
         sprintf (path, "/shaders/%s.%s.spv", shader_names[i], stage_names[j]);
 
-        if (!gulkan_renderer_create_shader_module (
-            gulkan_client_get_device_handle (GULKAN_CLIENT (self)), path,
-           &self->shader_modules[i * 2 + j]))
-          return false;
+        if (!gulkan_renderer_create_shader_module (GULKAN_RENDERER (self),
+                                                   path,
+                                                  &self->shader_modules[i * 2 + j]))
+          return FALSE;
       }
-  return true;
+  return TRUE;
 }
 
 /* Create a descriptor set layout compatible with all shaders. */
-static bool
+static gboolean
 _init_descriptor_layout (XrdSceneRenderer *self)
 {
   VkDescriptorSetLayoutCreateInfo info = {
@@ -250,16 +247,17 @@ _init_descriptor_layout (XrdSceneRenderer *self)
     }
   };
 
-  VkDevice device = gulkan_client_get_device_handle (GULKAN_CLIENT (self));
+  GulkanClient *gc = gxr_context_get_gulkan (self->context);
+  VkDevice device = gulkan_client_get_device_handle (gc);
   VkResult res = vkCreateDescriptorSetLayout (device,
                                              &info, NULL,
                                              &self->descriptor_set_layout);
-  vk_check_error ("vkCreateDescriptorSetLayout", res, false)
+  vk_check_error ("vkCreateDescriptorSetLayout", res, FALSE);
 
-  return true;
+  return TRUE;
 }
 
-static bool
+static gboolean
 _init_pipeline_layout (XrdSceneRenderer *self)
 {
   VkPipelineLayoutCreateInfo info = {
@@ -270,26 +268,29 @@ _init_pipeline_layout (XrdSceneRenderer *self)
     .pPushConstantRanges = NULL
   };
 
-  VkResult res = vkCreatePipelineLayout (gulkan_client_get_device_handle (
-                                           GULKAN_CLIENT (self)),
-                                        &info, NULL, &self->pipeline_layout);
-  vk_check_error ("vkCreatePipelineLayout", res, false)
+  GulkanClient *gc = gxr_context_get_gulkan (self->context);
 
-  return true;
+  VkResult res = vkCreatePipelineLayout (gulkan_client_get_device_handle (gc),
+                                        &info, NULL, &self->pipeline_layout);
+  vk_check_error ("vkCreatePipelineLayout", res, FALSE);
+
+  return TRUE;
 }
 
-static bool
+static gboolean
 _init_pipeline_cache (XrdSceneRenderer *self)
 {
   VkPipelineCacheCreateInfo info = {
     .sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO
   };
-  VkResult res = vkCreatePipelineCache (gulkan_client_get_device_handle (
-                                          GULKAN_CLIENT (self)),
-                                       &info, NULL, &self->pipeline_cache);
-  vk_check_error ("vkCreatePipelineCache", res, false)
 
-  return true;
+  GulkanClient *gc = gxr_context_get_gulkan (self->context);
+
+  VkResult res = vkCreatePipelineCache (gulkan_client_get_device_handle (gc),
+                                       &info, NULL, &self->pipeline_cache);
+  vk_check_error ("vkCreatePipelineCache", res, FALSE);
+
+  return TRUE;
 }
 
 typedef struct __attribute__((__packed__)) {
@@ -302,7 +303,7 @@ typedef struct __attribute__((__packed__)) {
   const VkPipelineRasterizationStateCreateInfo *rasterization_state;
 } XrdPipelineConfig;
 
-static bool
+static gboolean
 _init_graphics_pipelines (XrdSceneRenderer *self)
 {
   XrdPipelineConfig config[PIPELINE_COUNT] = {
@@ -445,11 +446,13 @@ _init_graphics_pipelines (XrdSceneRenderer *self)
     // PIPELINE_DEVICE_MODELS
     {
       .topology = VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST,
-      .stride = openvr_model_get_vertex_stride (),
+      .stride = gxr_context_get_model_vertex_stride (self->context),
       .attribs = (VkVertexInputAttributeDescription []) {
         {0, 0, VK_FORMAT_R32G32B32_SFLOAT, 0},
-        {1, 0, VK_FORMAT_R32G32B32_SFLOAT, openvr_model_get_normal_offset ()},
-        {2, 0, VK_FORMAT_R32G32_SFLOAT, openvr_model_get_uv_offset ()},
+        {1, 0, VK_FORMAT_R32G32B32_SFLOAT,
+          gxr_context_get_model_normal_offset (self->context)},
+        {2, 0, VK_FORMAT_R32G32_SFLOAT,
+          gxr_context_get_model_uv_offset (self->context)},
       },
       .depth_stencil_state = &(VkPipelineDepthStencilStateCreateInfo) {
           .sType = VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO,
@@ -500,7 +503,7 @@ _init_graphics_pipelines (XrdSceneRenderer *self)
         .pRasterizationState = config[i].rasterization_state,
         .pMultisampleState = &(VkPipelineMultisampleStateCreateInfo) {
           .sType = VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO,
-          .rasterizationSamples = self->msaa_sample_count,
+          .rasterizationSamples = self->sample_count,
           .minSampleShading = 0.0f,
           .pSampleMask = &(uint32_t) { 0xFFFFFFFF },
           .alphaToCoverageEnable = VK_FALSE
@@ -528,8 +531,7 @@ _init_graphics_pipelines (XrdSceneRenderer *self)
             .pName = "main"
           }
         },
-        .renderPass = gulkan_frame_buffer_get_render_pass (
-          self->framebuffer[GXR_EYE_LEFT]),
+        .renderPass = gulkan_render_pass_get_handle (self->render_pass),
         .pDynamicState = &(VkPipelineDynamicStateCreateInfo) {
           .sType = VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO,
           .dynamicStateCount = 2,
@@ -541,65 +543,66 @@ _init_graphics_pipelines (XrdSceneRenderer *self)
         .subpass = VK_NULL_HANDLE
       };
 
-      VkDevice device = gulkan_client_get_device_handle (GULKAN_CLIENT (self));
+      GulkanClient *gc = gxr_context_get_gulkan (self->context);
+
+      VkDevice device = gulkan_client_get_device_handle (gc);
       VkResult res;
       res = vkCreateGraphicsPipelines (device, self->pipeline_cache, 1,
                                       &pipeline_info, NULL,
                                       &self->pipelines[i]);
-      vk_check_error ("vkCreateGraphicsPipelines", res, false)
+      vk_check_error ("vkCreateGraphicsPipelines", res, FALSE);
     }
 
-  return true;
+  return TRUE;
 }
 
-
-static bool
-_init_vulkan (XrdSceneRenderer *self)
+gboolean
+xrd_scene_renderer_init_vulkan (XrdSceneRenderer *self,
+                                GxrContext       *context)
 {
+  if (self->context)
+    {
+      g_warning ("Scene Renderer: Vulkan already initialized.\n");
+      g_object_unref (self->context);
+    }
+
+  self->context = context;
+  g_object_ref (self->context);
+
+  gulkan_renderer_set_client (GULKAN_RENDERER (self),
+                              gxr_context_get_gulkan (context));
+
+  if (gxr_context_get_api (context) == GXR_API_OPENVR)
+    self->sample_count = VK_SAMPLE_COUNT_4_BIT;
+  else
+    self->sample_count = VK_SAMPLE_COUNT_1_BIT;
+
   if (!_init_framebuffers (self))
-    return false;
+    return FALSE;
 
   if (!_init_shaders (self))
-    return false;
+    return FALSE;
 
-  GulkanDevice *device = gulkan_client_get_device (GULKAN_CLIENT (self));
-  if (!gulkan_uniform_buffer_allocate_and_map (self->lights_buffer,
-                                               device, sizeof (XrdSceneLights)))
-    return false;
+
+  GulkanClient *gc = gxr_context_get_gulkan (context);
+  GulkanDevice *device = gulkan_client_get_device (gc);
+
+  self->lights_buffer =
+    gulkan_uniform_buffer_new (device, sizeof (XrdSceneLights));
+
+  if (!self->lights_buffer)
+    return FALSE;
 
   if (!_init_descriptor_layout (self))
-    return false;
+    return FALSE;
   if (!_init_pipeline_layout (self))
-    return false;
+    return FALSE;
   if (!_init_pipeline_cache (self))
-    return false;
+    return FALSE;
   if (!_init_graphics_pipelines (self))
-    return false;
+    return FALSE;
 
-  return true;
-}
-
-bool
-xrd_scene_renderer_init_vulkan_simple (XrdSceneRenderer *self)
-{
-  gulkan_client_init_vulkan (GULKAN_CLIENT (self), NULL, NULL);
-
-  if (!_init_vulkan (self))
-    return false;
-
-  return true;
-}
-
-bool
-xrd_scene_renderer_init_vulkan_openvr (XrdSceneRenderer *self)
-{
-  if (!openvr_compositor_gulkan_client_init (GULKAN_CLIENT (self)))
-    return false;
-
-  if (!_init_vulkan (self))
-    return false;
-
-  return true;
+  return TRUE;
 }
 
 VkDescriptorSetLayout *
@@ -611,24 +614,50 @@ xrd_scene_renderer_get_descriptor_set_layout (XrdSceneRenderer *self)
 static void
 _render_stereo (XrdSceneRenderer *self, VkCommandBuffer cmd_buffer)
 {
+  VkExtent2D extent = gulkan_renderer_get_extent (GULKAN_RENDERER (self));
+
+  gboolean flipped_viewport = FALSE;
+
   VkViewport viewport = {
-    0.0f, 0.0f,
-    self->render_width, self->render_height,
-    0.0f, 1.0f
+    .x = 0.0f,
+    .y = flipped_viewport ? (float) extent.height : 0.0f,
+    .width = (float) extent.width,
+    .height = (float) (flipped_viewport ? - extent.height : extent.height),
+    .minDepth = 0.0f,
+    .maxDepth = 1.0f
   };
   vkCmdSetViewport (cmd_buffer, 0, 1, &viewport);
   VkRect2D scissor = {
     .offset = {0, 0},
-    .extent = {self->render_width, self->render_height}
+    .extent = extent
   };
   vkCmdSetScissor (cmd_buffer, 0, 1, &scissor);
 
-  for (uint32_t eye = 0; eye < 2; eye++)
+  VkClearColorValue black = {
+    .float32 = { 0.0f, 0.0f, 0.0f, 1.0f },
+  };
+
+  uint32_t view_count = gxr_context_get_view_count (self->context);
+  for (uint32_t view = 0; view < view_count; view++)
     {
-      gulkan_frame_buffer_begin_pass (self->framebuffer[eye], cmd_buffer);
+      /* TOOD: adjust code to support rendering more than 2 views */
+      if (view >= 2)
+        break;
+
+      GulkanFrameBuffer *framebuffer =
+        gxr_context_get_acquired_framebuffer (self->context, view);
+
+      if (!GULKAN_IS_FRAME_BUFFER (framebuffer))
+        {
+          g_printerr ("framebuffer invalid for view %d\n", view);
+          continue;
+        }
+
+      gulkan_render_pass_begin (self->render_pass, extent, black,
+                                framebuffer, cmd_buffer);
 
       if (self->render_eye)
-        self->render_eye (eye, cmd_buffer, self->pipeline_layout,
+        self->render_eye (view, cmd_buffer, self->pipeline_layout,
                           self->pipelines, self->scene_client);
 
       vkCmdEndRenderPass (cmd_buffer);
@@ -637,9 +666,9 @@ _render_stereo (XrdSceneRenderer *self, VkCommandBuffer cmd_buffer)
 
 void
 xrd_scene_renderer_update_lights (XrdSceneRenderer  *self,
-                                  GList             *controllers)
+                                  GSList            *controllers)
 {
-  self->lights.active_lights = (int) g_list_length (controllers);
+  self->lights.active_lights = (int) g_slist_length (controllers);
   if (self->lights.active_lights > 2)
     {
       g_warning ("Update lights received more than 2 controllers.\n");
@@ -648,11 +677,11 @@ xrd_scene_renderer_update_lights (XrdSceneRenderer  *self,
 
   for (int i = 0; i < self->lights.active_lights; i++)
     {
-      GList *l = g_list_nth (controllers, (guint) i);
-      XrdController *controller = XRD_CONTROLLER (l->data);
+      GxrController *controller =
+        GXR_CONTROLLER (g_slist_nth_data (controllers, (guint) i));
 
       XrdScenePointerTip *scene_tip =
-        XRD_SCENE_POINTER_TIP (xrd_controller_get_pointer_tip (controller));
+        XRD_SCENE_POINTER_TIP (gxr_controller_get_pointer_tip (controller));
 
       graphene_point3d_t tip_position;
       xrd_scene_object_get_position (XRD_SCENE_OBJECT (scene_tip), &tip_position);
@@ -662,64 +691,50 @@ xrd_scene_renderer_update_lights (XrdSceneRenderer  *self,
       self->lights.lights[i].position[2] = tip_position.z;
     }
 
-  gulkan_uniform_buffer_update_struct (self->lights_buffer,
-                                       (gpointer) &self->lights);
+  gulkan_uniform_buffer_update (self->lights_buffer,
+                                (gpointer) &self->lights);
 }
 
 static void
 _draw (XrdSceneRenderer *self)
 {
-  GulkanCommandBuffer cmd_buffer;
-  gulkan_client_begin_cmd_buffer (GULKAN_CLIENT (self), &cmd_buffer);
+#if defined(RENDERDOC)
+  if (rdoc_api) rdoc_api->StartFrameCapture(NULL, NULL);
+  else _init_renderdoc ();
+#endif
+
+  GulkanClient *gc = gxr_context_get_gulkan (self->context);
+  GulkanDevice *device = gulkan_client_get_device (gc);
+
+  GulkanQueue *queue = gulkan_device_get_graphics_queue (device);
+
+  GulkanCmdBuffer *cmd_buffer = gulkan_queue_request_cmd_buffer (queue);
+  gulkan_cmd_buffer_begin (cmd_buffer);
 
   self->update_lights (self->scene_client);
 
-  _render_stereo (self, cmd_buffer.handle);
+  VkCommandBuffer cmd_handle = gulkan_cmd_buffer_get_handle (cmd_buffer);
 
-  vkEndCommandBuffer (cmd_buffer.handle);
+  _render_stereo (self, cmd_handle);
 
-  VkSubmitInfo submit_info = {
-    .sType = VK_STRUCTURE_TYPE_SUBMIT_INFO,
-    .commandBufferCount = 1,
-    .pCommandBuffers = &cmd_buffer.handle,
-    .waitSemaphoreCount = 0,
-    .pWaitSemaphores = NULL,
-    .signalSemaphoreCount = 0
-  };
+  gulkan_queue_submit (queue, cmd_buffer);
 
-  GulkanDevice *device = gulkan_client_get_device (GULKAN_CLIENT (self));
-  VkDevice device_handle = gulkan_device_get_handle (device);
-  vkQueueSubmit (gulkan_device_get_queue_handle (device), 1,
-                &submit_info, cmd_buffer.fence);
+  gulkan_queue_free_cmd_buffer (queue, cmd_buffer);
 
-  vkQueueWaitIdle (gulkan_device_get_queue_handle (device));
-
-  vkFreeCommandBuffers (device_handle,
-                        gulkan_client_get_command_pool (GULKAN_CLIENT (self)),
-                        1, &cmd_buffer.handle);
-  vkDestroyFence (device_handle, cmd_buffer.fence, NULL);
+#if defined(RENDERDOC)
+  if(rdoc_api) rdoc_api->EndFrameCapture(NULL, NULL);
+#endif
 }
 
-bool
+gboolean
 xrd_scene_renderer_draw (XrdSceneRenderer *self)
 {
   _draw (self);
 
-  VkImage left =
-    gulkan_frame_buffer_get_color_image (self->framebuffer[GXR_EYE_LEFT]);
+  if (!gxr_context_submit_framebuffers (self->context))
+    return FALSE;
 
-  VkImage right =
-    gulkan_frame_buffer_get_color_image (self->framebuffer[GXR_EYE_RIGHT]);
-
-  if (!openvr_compositor_submit (GULKAN_CLIENT(self),
-                                 self->render_width,
-                                 self->render_height,
-                                 VK_FORMAT_R8G8B8A8_UNORM,
-                                 self->msaa_sample_count,
-                                 left, right))
-    return false;
-
-  return true;
+  return TRUE;
 }
 
 void
@@ -744,15 +759,20 @@ xrd_scene_renderer_set_update_lights_cb (XrdSceneRenderer *self,
   self->scene_client = scene_client;
 }
 
-GulkanDevice*
-xrd_scene_renderer_get_device ()
-{
-  XrdSceneRenderer *self = xrd_scene_renderer_get_instance ();
-  return gulkan_client_get_device (GULKAN_CLIENT (self));
-}
-
 VkBuffer
 xrd_scene_renderer_get_lights_buffer_handle (XrdSceneRenderer *self)
 {
   return gulkan_uniform_buffer_get_handle (self->lights_buffer);
 }
+
+GulkanClient *
+xrd_scene_renderer_get_gulkan (XrdSceneRenderer *self)
+{
+  return gxr_context_get_gulkan (self->context);
+}
+
+GulkanRenderPass *
+xrd_scene_renderer_get_render_pass (XrdSceneRenderer *self)
+{
+  return self->render_pass;
+}
