@@ -25,12 +25,9 @@ G_BEGIN_DECLS
 G_DECLARE_FINAL_TYPE (XrdSceneSelection, xrd_scene_selection,
                       XRD, SCENE_SELECTION, XrdSceneObject)
 
-XrdSceneSelection *xrd_scene_selection_new (void);
-
-gboolean
-xrd_scene_selection_initialize (XrdSceneSelection     *self,
-                                GulkanDevice          *device,
-                                VkDescriptorSetLayout *layout);
+XrdSceneSelection *
+xrd_scene_selection_new (GulkanClient          *gulkan,
+                         VkDescriptorSetLayout *layout);
 
 void
 xrd_scene_selection_render (XrdSceneSelection *self,
