@@ -8,8 +8,9 @@
 #include <gulkan.h>
 
 #include "graphene-ext.h"
-#include "xrd-scene-window.h"
+#include "xrd-scene-window-private.h"
 #include "xrd-scene-renderer.h"
+#include "xrd-render-lock.h"
 
 enum
 {
@@ -26,15 +27,25 @@ enum
 static void
 xrd_scene_window_window_interface_init (XrdWindowInterface *iface);
 
-typedef struct {
+typedef struct __attribute__((__packed__)) {
   float color[4];
   bool flip_y;
-} XrdWindowUniformBuffer;
+} XrdWindowShadingUniformBuffer;
+
+typedef struct __attribute__((__packed__)) {
+  float mvp[16];
+  float mv[16];
+  float m[16];
+  bool receive_light;
+} XrdSceneWindowUniformBuffer;
 
 typedef struct _XrdSceneWindowPrivate
 {
   XrdSceneObject parent;
 
+  GulkanClient *gulkan;
+  VkBuffer lights;
+
   GulkanVertexBuffer *vertex_buffer;
   VkSampler sampler;
   float aspect_ratio;
@@ -43,7 +54,7 @@ typedef struct _XrdSceneWindowPrivate
   graphene_vec3_t color;
 
   GulkanUniformBuffer *shading_buffer;
-  XrdWindowUniformBuffer shading_buffer_data;
+  XrdWindowShadingUniformBuffer shading_buffer_data;
 
   XrdWindowData *window_data;
 } XrdSceneWindowPrivate;
@@ -163,8 +174,7 @@ xrd_scene_window_init (XrdSceneWindow *self)
   priv->sampler = VK_NULL_HANDLE;
   priv->aspect_ratio = 1.0;
   priv->window_data->texture = NULL;
-  priv->shading_buffer = gulkan_uniform_buffer_new ();
-  priv->shading_buffer_data.flip_y = false;
+  priv->shading_buffer_data.flip_y = FALSE;
 
   priv->window_data->title = NULL;
   priv->window_data->child_window = NULL;
@@ -175,23 +185,41 @@ xrd_scene_window_init (XrdSceneWindow *self)
   priv->window_data->texture = NULL;
   priv->window_data->xrd_window = XRD_WINDOW (self);
   priv->window_data->pinned = FALSE;
+
+  priv->window_data->owned_by_window = TRUE;
+
+  priv->gulkan = NULL;
+
   graphene_matrix_init_identity (&priv->window_data->reset_transform);
 }
 
-XrdSceneWindow *
-xrd_scene_window_new (const gchar *title)
+static XrdSceneWindow *
+_new (const gchar           *title,
+      GulkanClient          *gulkan,
+      VkDescriptorSetLayout *layout,
+      VkBuffer               lights)
 {
-  return (XrdSceneWindow*) g_object_new (XRD_TYPE_SCENE_WINDOW,
-                                         "title", title, NULL);
+  XrdSceneWindow* self = (XrdSceneWindow*) g_object_new (XRD_TYPE_SCENE_WINDOW,
+                                                         "title", title, NULL);
+  if (!xrd_scene_window_initialize (self, gulkan, layout, lights))
+    {
+      g_object_unref (self);
+      return NULL;
+    }
+
+  return self;
 }
 
 XrdSceneWindow *
-xrd_scene_window_new_from_meters (const gchar *title,
-                                  float        width,
-                                  float        height,
-                                  float        ppm)
+xrd_scene_window_new_from_meters (const gchar           *title,
+                                  float                  width,
+                                  float                  height,
+                                  float                  ppm,
+                                  GulkanClient          *gulkan,
+                                  VkDescriptorSetLayout *layout,
+                                  VkBuffer               lights)
 {
-  XrdSceneWindow *window = xrd_scene_window_new (title);
+  XrdSceneWindow *window = _new (title, gulkan, layout, lights);
   g_object_set (window,
                 "texture-width", (uint32_t) (width * ppm),
                 "texture-height", (uint32_t) (height * ppm),
@@ -206,9 +234,12 @@ _set_transformation (XrdWindow         *window,
                      graphene_matrix_t *mat);
 
 XrdSceneWindow *
-xrd_scene_window_new_from_data (XrdWindowData *data)
+xrd_scene_window_new_from_data (XrdWindowData         *data,
+                                GulkanClient          *gulkan,
+                                VkDescriptorSetLayout *layout,
+                                VkBuffer               lights)
 {
-  XrdSceneWindow *window = xrd_scene_window_new (data->title->str);
+  XrdSceneWindow *window = _new (data->title->str, gulkan, layout, lights);
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (window);
 
   // TODO: avoid unnecessary allocation
@@ -222,12 +253,15 @@ xrd_scene_window_new_from_data (XrdWindowData *data)
 }
 
 XrdSceneWindow *
-xrd_scene_window_new_from_pixels (const gchar *title,
-                                  uint32_t     width,
-                                  uint32_t     height,
-                                  float        ppm)
+xrd_scene_window_new_from_pixels (const gchar           *title,
+                                  uint32_t               width,
+                                  uint32_t               height,
+                                  float                  ppm,
+                                  GulkanClient          *gulkan,
+                                  VkDescriptorSetLayout *layout,
+                                  VkBuffer               lights)
 {
-  XrdSceneWindow *window = xrd_scene_window_new (title);
+  XrdSceneWindow *window = _new (title, gulkan, layout, lights);
   g_object_set (window,
                 "texture-width", width,
                 "texture-height", height,
@@ -238,16 +272,22 @@ xrd_scene_window_new_from_pixels (const gchar *title,
 }
 
 XrdSceneWindow *
-xrd_scene_window_new_from_native (const gchar *title,
-                                  gpointer     native,
-                                  uint32_t     width_pixels,
-                                  uint32_t     height_pixels,
-                                  float        ppm)
+xrd_scene_window_new_from_native (const gchar           *title,
+                                  gpointer               native,
+                                  uint32_t               width_pixels,
+                                  uint32_t               height_pixels,
+                                  float                  ppm,
+                                  GulkanClient          *gulkan,
+                                  VkDescriptorSetLayout *layout,
+                                  VkBuffer               lights)
 {
   XrdSceneWindow *window = xrd_scene_window_new_from_pixels (title,
                                                              width_pixels,
                                                              height_pixels,
-                                                             ppm);
+                                                             ppm,
+                                                             gulkan,
+                                                             layout,
+                                                             lights);
   g_object_set (window, "native", native, NULL);
   return window;
 }
@@ -259,14 +299,24 @@ xrd_scene_window_finalize (GObject *gobject)
   XrdSceneWindow *self = XRD_SCENE_WINDOW (gobject);
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
 
+  xrd_render_lock ();
   if (priv->window_data->texture)
     g_clear_object (&priv->window_data->texture);
+  xrd_render_unlock ();
 
-  XrdSceneRenderer *renderer = xrd_scene_renderer_get_instance ();
-  vkDestroySampler (gulkan_client_get_device_handle (GULKAN_CLIENT (renderer)),
+  vkDestroySampler (gulkan_client_get_device_handle (priv->gulkan),
                     priv->sampler, NULL);
-  g_object_unref (priv->vertex_buffer);
-  g_object_unref (priv->shading_buffer);
+  g_clear_object (&priv->vertex_buffer);
+  g_clear_object (&priv->shading_buffer);
+
+  g_clear_object (&priv->gulkan);
+
+  if (priv->window_data->owned_by_window)
+    {
+      if (priv->window_data->title)
+        g_string_free (priv->window_data->title, TRUE);
+      g_free (priv->window_data);
+    }
 
   G_OBJECT_CLASS (xrd_scene_window_parent_class)->finalize (gobject);
 }
@@ -284,26 +334,31 @@ _append_plane (GulkanVertexBuffer *vbo, float aspect_ratio)
 }
 
 gboolean
-xrd_scene_window_initialize (XrdSceneWindow *self)
+xrd_scene_window_initialize (XrdSceneWindow        *self,
+                             GulkanClient          *gulkan,
+                             VkDescriptorSetLayout *layout,
+                             VkBuffer               lights)
 {
-  XrdSceneObject *obj = XRD_SCENE_OBJECT (self);
-  XrdSceneRenderer *renderer = xrd_scene_renderer_get_instance ();
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
+  priv->gulkan = g_object_ref (gulkan);
+  priv->lights = lights;
 
-  GulkanDevice *device = gulkan_client_get_device (GULKAN_CLIENT (renderer));
+  GulkanDevice *device = gulkan_client_get_device (priv->gulkan);
 
   _append_plane (priv->vertex_buffer, priv->aspect_ratio);
   if (!gulkan_vertex_buffer_alloc_array (priv->vertex_buffer, device))
     return FALSE;
 
-  VkDescriptorSetLayout *layout =
-    xrd_scene_renderer_get_descriptor_set_layout (renderer);
+  VkDeviceSize ubo_size = sizeof (XrdSceneWindowUniformBuffer);
 
-  if (!xrd_scene_object_initialize (obj, layout))
+  XrdSceneObject *obj = XRD_SCENE_OBJECT (self);
+  if (!xrd_scene_object_initialize (obj, priv->gulkan, layout, ubo_size))
     return FALSE;
 
-  if (!gulkan_uniform_buffer_allocate_and_map (priv->shading_buffer,
-                                               device, sizeof (XrdWindowUniformBuffer)))
+  VkDeviceSize shading_ubo_size = sizeof (XrdWindowShadingUniformBuffer);
+
+  priv->shading_buffer = gulkan_uniform_buffer_new (device, shading_ubo_size);
+  if (!priv->shading_buffer)
     return FALSE;
 
   graphene_vec3_t white;
@@ -313,41 +368,53 @@ xrd_scene_window_initialize (XrdSceneWindow *self)
   return TRUE;
 }
 
-void
-xrd_scene_window_draw (XrdSceneWindow    *self,
-                       GxrEye             eye,
-                       VkPipeline         pipeline,
-                       VkPipelineLayout   pipeline_layout,
-                       VkCommandBuffer    cmd_buffer,
-                       graphene_matrix_t *vp)
+static void
+_update_ubo (XrdSceneWindow    *self,
+             GxrEye             eye,
+             graphene_matrix_t *view_matrix,
+             graphene_matrix_t *projection_matrix,
+             gboolean           shaded)
 {
-  XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
-  if (!priv->window_data->texture)
+  XrdSceneWindowUniformBuffer ub;
+
+  graphene_matrix_t m_matrix;
+  xrd_scene_object_get_transformation (XRD_SCENE_OBJECT (self), &m_matrix);
+
+  graphene_matrix_t mv_matrix;
+  graphene_matrix_multiply (&m_matrix, view_matrix, &mv_matrix);
+
+  graphene_matrix_t mvp_matrix;
+  graphene_matrix_multiply (&mv_matrix, projection_matrix, &mvp_matrix);
+
+  float mvp[16];
+  graphene_matrix_to_float (&mvp_matrix, mvp);
+
+  float mv[16];
+  graphene_matrix_to_float (&mv_matrix, mv);
+
+  float m[16];
+  graphene_matrix_to_float (&m_matrix, m);
+
+  for (int i = 0; i < 16; i++)
     {
-      /* g_warning ("Trying to draw window with no texture.\n"); */
-      return;
+      ub.mvp[i] = mvp[i];
+      ub.mv[i] = mv[i];
+      ub.m[i] = m[i];
+      ub.receive_light = shaded;
     }
 
-  XrdSceneObject *obj = XRD_SCENE_OBJECT (self);
-  if (!xrd_scene_object_is_visible (obj))
-    return;
-
-  vkCmdBindPipeline (cmd_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
-
-  xrd_scene_object_update_mvp_matrix (obj, eye, vp);
-
-  xrd_scene_object_bind (obj, eye, cmd_buffer, pipeline_layout);
-  gulkan_vertex_buffer_draw (priv->vertex_buffer, cmd_buffer);
+  xrd_scene_object_update_ubo (XRD_SCENE_OBJECT (self), eye, &ub);
 }
 
 void
-xrd_scene_window_draw_shaded (XrdSceneWindow    *self,
-                              GxrEye             eye,
-                              VkPipeline         pipeline,
-                              VkPipelineLayout   pipeline_layout,
-                              VkCommandBuffer    cmd_buffer,
-                              graphene_matrix_t *view,
-                              graphene_matrix_t *projection)
+xrd_scene_window_render (XrdSceneWindow    *self,
+                         GxrEye             eye,
+                         VkPipeline         pipeline,
+                         VkPipelineLayout   pipeline_layout,
+                         VkCommandBuffer    cmd_buffer,
+                         graphene_matrix_t *view,
+                         graphene_matrix_t *projection,
+                         gboolean           shaded)
 {
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
   if (!priv->window_data->texture)
@@ -360,9 +427,9 @@ xrd_scene_window_draw_shaded (XrdSceneWindow    *self,
   if (!xrd_scene_object_is_visible (obj))
     return;
 
-  vkCmdBindPipeline (cmd_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
+  _update_ubo (self, eye, view, projection, shaded);
 
-  xrd_scene_object_update_transformation_buffer (obj, eye, view, projection);
+  vkCmdBindPipeline (cmd_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipeline);
 
   xrd_scene_object_bind (obj, eye, cmd_buffer, pipeline_layout);
   gulkan_vertex_buffer_draw (priv->vertex_buffer, cmd_buffer);
@@ -378,19 +445,21 @@ xrd_scene_window_set_color (XrdSceneWindow        *self,
   graphene_vec4_t color_vec4;
   graphene_vec4_init_from_vec3 (&color_vec4, color, 1.0f);
 
-  graphene_vec4_to_float (&color_vec4, priv->shading_buffer_data.color);
+  float color_array[4];
+  graphene_vec4_to_float (&color_vec4, color_array);
 
-  gulkan_uniform_buffer_update_struct (priv->shading_buffer,
-                                       (gpointer) &priv->shading_buffer_data);
+  for (uint32_t i = 0; i < 4; i++)
+    priv->shading_buffer_data.color[i] = color_array[i];
+
+  gulkan_uniform_buffer_update (priv->shading_buffer,
+                                (gpointer) &priv->shading_buffer_data);
 }
 
 void
 xrd_scene_window_update_descriptors (XrdSceneWindow *self)
 {
-  XrdSceneRenderer *renderer = xrd_scene_renderer_get_instance ();
-  VkDevice device = gulkan_client_get_device_handle (GULKAN_CLIENT (renderer));
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
-  VkBuffer lights = xrd_scene_renderer_get_lights_buffer_handle (renderer);
+  VkDevice device = gulkan_client_get_device_handle (priv->gulkan);
 
   for (uint32_t eye = 0; eye < 2; eye++)
     {
@@ -449,7 +518,7 @@ xrd_scene_window_update_descriptors (XrdSceneWindow *self)
           .descriptorCount = 1,
           .descriptorType = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER,
           .pBufferInfo = &(VkDescriptorBufferInfo) {
-            .buffer = lights,
+            .buffer = priv->lights,
             .offset = 0,
             .range = VK_WHOLE_SIZE
           },
@@ -492,7 +561,7 @@ _get_transformation (XrdWindow         *window,
                      graphene_matrix_t *mat)
 {
   XrdSceneWindow *self = XRD_SCENE_WINDOW (window);
-  *mat = xrd_scene_object_get_transformation (XRD_SCENE_OBJECT (self));
+  xrd_scene_object_get_transformation (XRD_SCENE_OBJECT (self), mat);
   return TRUE;
 }
 
@@ -506,31 +575,30 @@ _get_transformation_no_scale (XrdWindow         *window,
 }
 
 static void
-_submit_texture (XrdWindow     *window,
-                 GulkanClient  *client)
+_submit_texture (XrdWindow *window)
 {
   (void) window;
-  (void) client;
 }
 
 static void
-_set_and_submit_texture (XrdWindow *window, GulkanClient *client,
-                         GulkanTexture *texture)
+_set_and_submit_texture (XrdWindow *window, GulkanTexture *texture)
 {
   XrdSceneWindow *self = XRD_SCENE_WINDOW (window);
   XrdSceneWindowPrivate *priv = xrd_scene_window_get_instance_private (self);
 
+  xrd_render_lock ();
+
   if (texture == priv->window_data->texture)
     {
       gchar *title;
       g_object_get (window, "title", &title, NULL);
       g_debug ("Texture %p was already set on window %p (%s).\n",
                (void*) texture, (void*) window, title);
+
+      xrd_render_unlock ();
       return;
     }
 
-  uint32_t w = gulkan_texture_get_width (texture);
-  uint32_t h = gulkan_texture_get_height (texture);
 
   uint32_t previous_texture_width, previous_Texture_height;
   g_object_get (self,
@@ -538,14 +606,15 @@ _set_and_submit_texture (XrdWindow *window, GulkanClient *client,
                 "texture-height", &previous_Texture_height,
                 NULL);
 
+  VkExtent2D extent = gulkan_texture_get_extent (texture);
   g_object_set (window,
-                "texture-width", w,
-                "texture-height", h,
+                "texture-width", extent.width,
+                "texture-height", extent.height,
                 NULL);
 
-  VkDevice device = gulkan_client_get_device_handle (client);
+  VkDevice device = gulkan_client_get_device_handle (priv->gulkan);
 
-  float aspect_ratio = (float) w / (float) h;
+  float aspect_ratio = (float) extent.width / (float) extent.height;
 
   if (priv->aspect_ratio != aspect_ratio)
     {
@@ -582,7 +651,8 @@ _set_and_submit_texture (XrdWindow *window, GulkanClient *client,
 
   xrd_scene_window_update_descriptors (self);
 
-  if (previous_texture_width != w || previous_Texture_height != h)
+  if (previous_texture_width != extent.width ||
+      previous_Texture_height != extent.height)
     {
       /* initial-dims are respective the texture size and ppm.
        * Now that the texture size changed, initial dims need to be
@@ -594,11 +664,13 @@ _set_and_submit_texture (XrdWindow *window, GulkanClient *client,
                     NULL);
 
       float previous_ppm = (float)previous_texture_width / initial_width_meter;
-      float new_initial_width_meter = (float)w / previous_ppm;
+      float new_initial_width_meter = (float) extent.width / previous_ppm;
 
       /* updates "initial-width-meters"  and "initial height-meters"! */
       xrd_scene_window_set_width_meters (self, new_initial_width_meter);
     }
+
+  xrd_render_unlock ();
 }
 
 static GulkanTexture *
@@ -635,8 +707,8 @@ _set_flip_y (XrdWindow *window,
   priv->flip_y = flip_y;
   priv->shading_buffer_data.flip_y = flip_y;
 
-  gulkan_uniform_buffer_update_struct (priv->shading_buffer,
-                                       (gpointer) &priv->shading_buffer_data);
+  gulkan_uniform_buffer_update (priv->shading_buffer,
+                                (gpointer) &priv->shading_buffer_data);
 }
 
 void
