@@ -24,7 +24,7 @@ struct _XrdContainer
   float distance;
 
   graphene_matrix_t transform;
-  XrdController *controller;
+  GxrController *controller;
 
   XrdContainerAttachment attachment;
   XrdContainerLayout layout;
@@ -266,7 +266,7 @@ _window_container_set_transformation (XrdContainer *self,
 }
 
 static gboolean
-_step_fov (XrdContainer *self)
+_step_fov (XrdContainer *self, GxrContext *context)
 {
   /* Containers outside fov_factor_outer * FOV size will be snapped to
    * fov_factor_outer * FOV size.
@@ -277,7 +277,7 @@ _step_fov (XrdContainer *self)
   const float fov_factor_inner = 0.25f;
 
   graphene_matrix_t hmd_pose;
-  openvr_system_get_hmd_pose (&hmd_pose);
+  gxr_context_get_head_pose (context, &hmd_pose);
   graphene_matrix_t hmd_pose_inv;
   graphene_matrix_inverse (&hmd_pose, &hmd_pose_inv);
 
@@ -291,7 +291,8 @@ _step_fov (XrdContainer *self)
   graphene_ext_matrix_get_translation_vec3 (&wc_transform_cs, &wc_vec_cs);
 
   float left, right, top, bottom;
-  openvr_system_get_frustum_angles (GXR_EYE_LEFT, &left, &right, &top, &bottom);
+  gxr_context_get_frustum_angles (context, GXR_EYE_LEFT,
+                                  &left, &right, &top, &bottom);
 
   float left_inner = left * fov_factor_inner;
   float right_inner = right * fov_factor_inner;
@@ -421,7 +422,7 @@ _step_fov (XrdContainer *self)
   float inclination_diff = inclination - intersection_azimuth_inclination.y;
 
   float ms_since_last_step =
-    (g_get_monotonic_time () - self->last_step_timestamp) / 1000.f;
+    (float) (g_get_monotonic_time () - self->last_step_timestamp) / 1000.f;
 
   /* Container moves by a fixed fraction of the distance from current point
    * to target point, i.e. on a linear deceleration curve. */
@@ -477,7 +478,7 @@ _step_hand (XrdContainer *self)
   graphene_matrix_translate (&container_transform, &offset);
 
   graphene_matrix_t controller_pose;
-  xrd_controller_get_pose_hand_grip (self->controller, &controller_pose);
+  gxr_controller_get_hand_grip_pose (self->controller, &controller_pose);
 
   graphene_matrix_multiply (&container_transform,
                             &controller_pose,
@@ -490,20 +491,21 @@ _step_hand (XrdContainer *self)
 /**
  * xrd_container_step:
  * @self: The #XrdContainer
+ * @context: A #GxrContext
  *
  * Updates the container's position based on its attachment.
  *
  * Returns: A #gboolean if that is %TRUE the step was successful.
  */
 gboolean
-xrd_container_step (XrdContainer *self)
+xrd_container_step (XrdContainer *self, GxrContext *context)
 {
   gboolean ret = FALSE;
   switch (self->attachment)
   {
     case (XRD_CONTAINER_ATTACHMENT_HEAD):
     {
-      ret = _step_fov (self);
+      ret = _step_fov (self, context);
       break;
     }
     case (XRD_CONTAINER_ATTACHMENT_HAND):
@@ -532,7 +534,7 @@ xrd_container_step (XrdContainer *self)
 void
 xrd_container_set_attachment (XrdContainer *self,
                               XrdContainerAttachment attachment,
-                              XrdController *controller)
+                              GxrController *controller)
 {
   self->attachment = attachment;
   self->controller = controller;
@@ -594,15 +596,22 @@ xrd_container_is_visible (XrdContainer *self)
 /**
  * xrd_container_center_view:
  * @self: The container.
+ * @context: A #GxrContext
  * @distance: The distance from the HMD the container should have.
  *
  * Places the container in the center of the FOV at the given distance.
  */
 void
-xrd_container_center_view (XrdContainer *self, float distance)
+xrd_container_center_view (XrdContainer *self,
+                           GxrContext *context,
+                           float distance)
 {
   graphene_matrix_t hmd_transform;
-  openvr_system_get_hmd_pose (&hmd_transform);
+  if (!gxr_context_get_head_pose (context, &hmd_transform))
+    {
+      g_printerr ("Could not get head pose.\n");
+      return;
+    }
 
   graphene_point3d_t distance_point = {
       .x = 0,
