@@ -6,7 +6,6 @@
  */
 
 #include <gdk/gdk.h>
-#include <math.h>
 
 #include <gxr.h>
 
@@ -14,7 +13,11 @@
 #include "xrd-math.h"
 #include "graphene-ext.h"
 
-#include "xrd-controller.h"
+#include "xrd-render-lock.h"
+
+#ifndef M_PI
+#define M_PI (3.14159265358979323846)
+#endif
 
 struct _XrdWindowManager
 {
@@ -98,6 +101,7 @@ xrd_window_manager_finalize (GObject *gobject)
   g_slist_free (self->hoverable_windows);
   g_slist_free (self->containers);
   g_slist_free (self->draggable_windows);
+  g_slist_free (self->managed_windows);
 
   g_slist_free_full (self->destroy_windows, g_object_unref);
 }
@@ -116,6 +120,8 @@ _interpolate_cb (gpointer _transition)
                                           &transition->to,
                                            curve,
                                           &interpolated);
+
+  xrd_render_lock ();
   xrd_window_set_transformation (window, &interpolated);
 
   float interpolated_scaling =
@@ -125,7 +131,7 @@ _interpolate_cb (gpointer _transition)
   g_object_set (G_OBJECT(window), "scale", (double) interpolated_scaling, NULL);
 
   gint64 now = g_get_monotonic_time ();
-  float ms_since_last = (now - transition->last_timestamp) / 1000.f;
+  float ms_since_last = (float) (now - transition->last_timestamp) / 1000.f;
   transition->last_timestamp = now;
 
   /* in seconds */
@@ -142,9 +148,12 @@ _interpolate_cb (gpointer _transition)
 
       g_object_unref (transition->window);
       g_free (transition);
+
+      xrd_render_unlock ();
       return FALSE;
     }
 
+  xrd_render_unlock ();
   return TRUE;
 }
 
@@ -220,7 +229,8 @@ _azimuth_from_pose (graphene_matrix_t *mat)
 }
 
 gboolean
-xrd_window_manager_arrange_sphere (XrdWindowManager *self)
+xrd_window_manager_arrange_sphere (XrdWindowManager *self,
+                                   GxrContext       *context)
 {
   guint num_overlays = g_slist_length (self->managed_windows);
 
@@ -233,7 +243,12 @@ xrd_window_manager_arrange_sphere (XrdWindowManager *self)
     grid_width++;
 
   graphene_matrix_t hmd_pose;
-  openvr_system_get_hmd_pose (&hmd_pose);
+  if (!gxr_context_get_head_pose (context, &hmd_pose))
+    {
+      g_printerr ("Could not get head pose./n");
+      return FALSE;
+    }
+
   graphene_vec3_t hmd_vec;
   graphene_ext_matrix_get_translation_vec3 (&hmd_pose, &hmd_vec);
 
@@ -343,6 +358,7 @@ xrd_window_manager_add_window (XrdWindowManager *self,
                                XrdWindow *window,
                                XrdWindowFlags flags)
 {
+  xrd_render_lock ();
   /* any window must be either in all_windows or buttons */
   if (flags & XRD_WINDOW_BUTTON)
     {
@@ -373,10 +389,12 @@ xrd_window_manager_add_window (XrdWindowManager *self,
 
   /* keep the window referenced as long as the window manages this window */
   g_object_ref (window);
+  xrd_render_unlock ();
 }
 
 void
-xrd_window_manager_poll_window_events (XrdWindowManager *self)
+xrd_window_manager_poll_window_events (XrdWindowManager *self,
+                                       GxrContext       *context)
 {
   for (GSList *l = self->hoverable_windows; l != NULL; l = l->next)
     {
@@ -387,7 +405,7 @@ xrd_window_manager_poll_window_events (XrdWindowManager *self)
   for (GSList *l = self->containers; l != NULL; l = l->next)
     {
       XrdContainer *wc = (XrdContainer *) l->data;
-      xrd_container_step (wc);
+      xrd_container_step (wc, context);
     }
 }
 
@@ -412,17 +430,45 @@ xrd_window_manager_remove_window (XrdWindowManager *self,
   g_object_unref (window);
 }
 
-static void
-_test_hover (XrdWindowManager  *self,
-             graphene_matrix_t *pose,
-             XrdController     *controller)
+/* TODO: move somewhere better */
+static gboolean
+_controller_intersects (GxrController *controller,
+                        XrdWindow *window,
+                        graphene_point3d_t *intersection_point)
 {
-  XrdHoverEvent *hover_event = g_malloc (sizeof (XrdHoverEvent));
-  hover_event->distance = FLT_MAX;
+  GxrPointer *p = gxr_controller_get_pointer (controller);
 
+  graphene_plane_t plane;
+  xrd_window_get_plane (window, &plane);
+
+  graphene_matrix_t window_transform;
+  xrd_window_get_transformation (window, &window_transform);
+
+  float aspect_ratio = xrd_window_get_aspect_ratio (window);
+
+  float distance;
+  graphene_vec3_t intersection_vec;
+
+  gboolean intersects =
+    gxr_pointer_get_plane_intersection (p, &plane, &window_transform,
+                                        aspect_ratio, &distance,
+                                        &intersection_vec);
+
+  if (intersects)
+    graphene_point3d_init_from_vec3 (intersection_point, &intersection_vec);
+
+  return intersects;
+}
+
+static XrdWindow *
+_get_hover_target (XrdWindowManager  *self,
+                   GxrController *controller,
+                   graphene_point3d_t *intersection_point,
+                   float *intersection_distance)
+{
   XrdWindow *closest = NULL;
-
-  XrdPointer *pointer = xrd_controller_get_pointer (controller);
+  *intersection_distance = FLT_MAX;
+  graphene_point3d_t point;
 
   for (GSList *l = self->hoverable_windows; l != NULL; l = l->next)
     {
@@ -435,203 +481,76 @@ _test_hover (XrdWindowManager  *self,
         if (g_slist_find (self->buttons, window) == NULL)
           continue;
 
-      graphene_point3d_t intersection_point;
-      if (xrd_window_intersects (window, pointer, pose, &intersection_point))
+      if (_controller_intersects (controller, window, &point))
         {
-          float distance =
-            xrd_math_point_matrix_distance (&intersection_point, pose);
-          if (distance < hover_event->distance)
+          float distance = gxr_controller_get_distance (controller, &point);
+          if (distance < *intersection_distance)
             {
               closest = window;
-              hover_event->distance = distance;
-              graphene_matrix_init_from_matrix (&hover_event->pose, pose);
-              graphene_point3d_init_from_point (&hover_event->point,
-                                                &intersection_point);
+              *intersection_distance = distance;
+              graphene_point3d_init_from_point (intersection_point, &point);
             }
         }
     }
-
-  XrdHoverState *hover_state = xrd_controller_get_hover_state (controller);
-
-  xrd_pointer_set_selected_window (pointer, closest);
-
-  if (closest != NULL)
-    {
-      /* The recipient of the hover_end event should already see that this
-       * overlay is not hovered anymore, so we need to set the hover state
-       * before sending the event */
-      XrdWindow *last_hovered_window = hover_state->window;
-      hover_state->distance = hover_event->distance;
-      hover_state->window = closest;
-      graphene_matrix_init_from_matrix (&hover_state->pose, pose);
-
-      /* We now hover over an overlay */
-      if (closest != last_hovered_window)
-        {
-          XrdControllerIndexEvent *hover_start_event =
-              g_malloc (sizeof (XrdControllerIndexEvent));
-          hover_start_event->controller_handle =
-            xrd_controller_get_handle (controller);
-          xrd_window_emit_hover_start (closest, hover_start_event);
-        }
-
-      if (closest != last_hovered_window
-          && last_hovered_window != NULL)
-        {
-          XrdControllerIndexEvent *hover_end_event =
-              g_malloc (sizeof (XrdControllerIndexEvent));
-          hover_end_event->controller_handle =
-            xrd_controller_get_handle (controller);
-          xrd_window_emit_hover_end (last_hovered_window, hover_end_event);
-        }
-
-      xrd_window_get_intersection_2d (
-        closest, &hover_event->point, &hover_state->intersection_2d);
-
-      hover_event->controller_handle = xrd_controller_get_handle (controller);
-      xrd_window_emit_hover (closest, hover_event);
-    }
-  else
-    {
-      /* No intersection was found, nothing is hovered */
-      g_free (hover_event);
-
-      /* Emit hover end event only if we had hovered something earlier */
-      if (hover_state->window != NULL)
-        {
-          XrdWindow *last_hovered_window = hover_state->window;
-          xrd_controller_reset_hover_state (controller);
-          XrdControllerIndexEvent *hover_end_event =
-              g_malloc (sizeof (XrdControllerIndexEvent));
-          hover_end_event->controller_handle =
-            xrd_controller_get_handle (controller);
-          xrd_window_emit_hover_end (last_hovered_window, hover_end_event);
-        }
-
-      /* Emit no hover event every time when hovering nothing */
-      XrdNoHoverEvent *no_hover_event = g_malloc (sizeof (XrdNoHoverEvent));
-      no_hover_event->controller_handle =
-        xrd_controller_get_handle (controller);
-      graphene_matrix_init_from_matrix (&no_hover_event->pose, pose);
-      g_signal_emit (self, manager_signals[NO_HOVER_EVENT], 0, no_hover_event);
-    }
+  return closest;
 }
 
 static void
-_drag_window (XrdWindowManager  *self,
-              graphene_matrix_t *pose,
-              XrdController     *controller)
+_emit_hover (XrdWindowManager   *self,
+             GxrController      *controller,
+             XrdWindow          *last_window,
+             XrdWindow          *window,
+             graphene_point3d_t *intersection_point,
+             float               intersection_distance)
 {
-  (void) self;
-  XrdHoverState *hover_state = xrd_controller_get_hover_state (controller);
-  XrdGrabState *grab_state = xrd_controller_get_grab_state (controller);
-
-  graphene_point3d_t controller_translation_point;
-  graphene_ext_matrix_get_translation_point3d (pose,
-                                              &controller_translation_point);
-  graphene_quaternion_t controller_rotation;
-  graphene_quaternion_init_from_matrix (&controller_rotation, pose);
-
-  graphene_point3d_t distance_translation_point;
-  graphene_point3d_init (&distance_translation_point,
-                         0.f, 0.f, -hover_state->distance);
-
-  /* Build a new transform for pointer tip in event->pose.
-   * Pointer tip is at intersection, in the plane of the window,
-   * so we can reuse the tip rotation for the window rotation. */
-  XrdGrabEvent *event = g_malloc (sizeof (XrdGrabEvent));
-  event->controller_handle = xrd_controller_get_handle (controller);
-  graphene_matrix_init_identity (&event->pose);
-
-  /* restore original rotation of the tip */
-  graphene_matrix_rotate_quaternion (&event->pose,
-                                     &grab_state->window_rotation);
-
-  /* Later the current controller rotation is applied to the overlay, so to
-   * keep the later controller rotations relative to the initial controller
-   * rotation, rotate the window in the opposite direction of the initial
-   * controller rotation.
-   * This will initially result in the same window rotation so the window does
-   * not change its rotation when being grabbed, and changing the controllers
-   * position later will rotate the window with the "diff" of the controller
-   * rotation to the initial controller rotation. */
-  graphene_matrix_rotate_quaternion (
-      &event->pose, &grab_state->inverse_controller_rotation);
-
-  /* then translate the overlay to the controller ray distance */
-  graphene_matrix_translate (&event->pose, &distance_translation_point);
-
-  /* Rotate the translated overlay to where the controller is pointing. */
-  graphene_matrix_rotate_quaternion (&event->pose,
-                                     &controller_rotation);
-
-  /* Calculation was done for controller in (0,0,0), just move it with
-   * controller's offset to real (0,0,0) */
-  graphene_matrix_translate (&event->pose, &controller_translation_point);
+  if (window)
+    {
+      /* Emit hover event every time when hovering a window */
+      XrdHoverEvent *hover_event = g_malloc (sizeof (XrdHoverEvent));
+      hover_event->distance = intersection_distance;
+      graphene_point3d_init_from_point(&hover_event->point, intersection_point);
+      hover_event->controller = controller;
+      xrd_window_emit_hover (window, hover_event);
 
 
+      /* Emit hover start event when starting to hover a new window */
+      if (window != last_window)
+        xrd_window_emit_hover_start (window, controller);
 
-  graphene_matrix_t transformation_matrix;
-  graphene_matrix_init_identity (&transformation_matrix);
+      /* Emit hover end event when hovering over a new window */
+      if (last_window && window != last_window)
+        xrd_window_emit_hover_end (last_window, controller);
+    }
+  else
+    {
+      /* Emit no hover event every time when hovering nothing */
+      XrdNoHoverEvent *no_hover_event = g_malloc (sizeof (XrdNoHoverEvent));
+      no_hover_event->controller = controller;
+      g_signal_emit (self, manager_signals[NO_HOVER_EVENT], 0, no_hover_event);
 
-  /* translate such that the grab point is pivot point. */
-  graphene_matrix_translate (&transformation_matrix,
-                             &grab_state->grab_offset);
 
-  /* window has the same rotation as the tip we calculated in event->pose */
-  graphene_matrix_multiply (&transformation_matrix,
-                            &event->pose,
-                            &transformation_matrix);
-
-  xrd_window_set_transformation (grab_state->window,
-                                 &transformation_matrix);
-
-  xrd_window_emit_grab (grab_state->window, event);
-
-  XrdPointer *pointer = xrd_controller_get_pointer (controller);
-  xrd_pointer_set_selected_window (pointer, grab_state->window);
+      /* Emit hover end event only if we had hovered something earlier */
+      if (last_window != NULL)
+        {
+          gxr_controller_reset_hover_state (controller);
+          xrd_window_emit_hover_end (last_window, controller);
+        }
+    }
 }
 
 void
 xrd_window_manager_drag_start (XrdWindowManager *self,
-                               XrdController    *controller)
+                               GxrController    *controller)
 {
-  XrdHoverState *hover_state = xrd_controller_get_hover_state (controller);
-  XrdGrabState *grab_state = xrd_controller_get_grab_state (controller);
-
-  if (!_is_in_list (self->draggable_windows, hover_state->window))
+  GxrHoverState *hover_state = gxr_controller_get_hover_state (controller);
+  if (!_is_in_list (self->draggable_windows, hover_state->hovered_object))
     return;
 
-  /* Copy hover to grab state */
-  grab_state->window = hover_state->window;
+  XrdWindow *window = XRD_WINDOW (hover_state->hovered_object);
+  graphene_matrix_t window_pose;
+  xrd_window_get_transformation_no_scale (window, &window_pose);
 
-  graphene_quaternion_t controller_rotation;
-  graphene_ext_matrix_get_rotation_quaternion (&hover_state->pose,
-                                               &controller_rotation);
-
-  graphene_matrix_t window_transform;
-  xrd_window_get_transformation_no_scale (grab_state->window, &window_transform);
-
-  graphene_ext_matrix_get_rotation_quaternion (&window_transform,
-                                               &grab_state->window_rotation);
-
-  graphene_point3d_t distance_translation_point;
-  graphene_point3d_init (&distance_translation_point,
-                         0.f, 0.f, -hover_state->distance);
-
-  graphene_point3d_t negative_distance_translation_point;
-  graphene_point3d_init (&negative_distance_translation_point,
-                         0.f, 0.f, +hover_state->distance);
-
-  graphene_point3d_init (
-      &grab_state->grab_offset,
-      -hover_state->intersection_2d.x,
-      -hover_state->intersection_2d.y,
-      0.f);
-
-  graphene_quaternion_invert (
-      &controller_rotation,
-      &grab_state->inverse_controller_rotation);
+  gxr_controller_drag_start (controller, window, &window_pose);
 }
 
 /* checks if a float is in the range specified when the property was created */
@@ -658,75 +577,128 @@ _valid_float_prop (GObject *object, const gchar *prop, float value)
  */
 void
 xrd_window_manager_scale (XrdWindowManager *self,
-                          XrdGrabState     *grab_state,
+                          GxrGrabState     *grab_state,
                           float             factor,
                           float             update_rate_ms)
 {
-  if (grab_state->window == NULL)
+  if (grab_state->grabbed_object == NULL)
     return;
   (void) self;
 
   float current_factor;
-  g_object_get (G_OBJECT(grab_state->window), "scale", &current_factor, NULL);
+  g_object_get (G_OBJECT(grab_state->grabbed_object),
+                "scale", &current_factor,
+                NULL);
 
   float new_factor = current_factor +
                      current_factor * factor * (update_rate_ms / 1000.f);
 
-  if (!_valid_float_prop (G_OBJECT (grab_state->window), "scale", new_factor))
-      return;
+  if (!_valid_float_prop (G_OBJECT (grab_state->grabbed_object),
+    "scale", new_factor))
+    return;
 
   /* Grab point is relative to overlay center so we can just scale it */
-  graphene_point3d_scale (&grab_state->grab_offset,
-                          1 + factor * (update_rate_ms / 1000.f),
-                          &grab_state->grab_offset);
 
-  g_object_set (G_OBJECT(grab_state->window),
-                "scale", (double) new_factor, NULL);
+  float scale = 1 + factor * (update_rate_ms / 1000.f);
+  grab_state->grab_offset.x = grab_state->grab_offset.x * scale;
+  grab_state->grab_offset.y = grab_state->grab_offset.y * scale;
+
+  g_object_set (G_OBJECT(grab_state->grabbed_object),
+                "scale", (double) new_factor,
+                NULL);
 }
 
 void
 xrd_window_manager_check_grab (XrdWindowManager *self,
-                               XrdController    *controller)
+                               GxrController    *controller)
 {
   (void) self;
-  XrdHoverState *hover_state = xrd_controller_get_hover_state (controller);
+  GxrHoverState *hover_state = gxr_controller_get_hover_state (controller);
 
-  if (hover_state->window == NULL)
+  if (hover_state->hovered_object == NULL)
     return;
 
-   XrdControllerIndexEvent *grab_event =
-      g_malloc (sizeof (XrdControllerIndexEvent));
-  grab_event->controller_handle = xrd_controller_get_handle (controller);
-  xrd_window_emit_grab_start (hover_state->window, grab_event);
+  xrd_window_emit_grab_start (hover_state->hovered_object, controller);
 }
 
 void
 xrd_window_manager_check_release (XrdWindowManager *self,
-                                  XrdController    *controller)
+                                  GxrController    *controller)
 {
   (void) self;
-  XrdGrabState *grab_state = xrd_controller_get_grab_state (controller);
+  GxrGrabState *grab_state = gxr_controller_get_grab_state (controller);
 
-  if (grab_state->window == NULL)
+  if (grab_state->grabbed_object == NULL)
     return;
 
-  XrdControllerIndexEvent *release_event =
-      g_malloc (sizeof (XrdControllerIndexEvent));
-  release_event->controller_handle = xrd_controller_get_handle (controller);
-  xrd_window_emit_release (grab_state->window, release_event);
-  xrd_controller_reset_grab_state (controller);
+  xrd_window_emit_release (grab_state->grabbed_object, controller);
+  gxr_controller_reset_grab_state (controller);
 }
 
 void
-xrd_window_manager_update_pose (XrdWindowManager  *self,
-                                graphene_matrix_t *pose,
-                                XrdController     *controller)
+xrd_window_manager_update_controller (XrdWindowManager  *self,
+                                      GxrController     *controller)
 {
   /* Drag test */
-  if (xrd_controller_get_grab_state (controller)->window != NULL)
-    _drag_window (self, pose, controller);
+  gpointer grabbed_object =
+    gxr_controller_get_grab_state (controller)->grabbed_object;
+
+  if (grabbed_object != NULL)
+    {
+      XrdWindow *window = XRD_WINDOW (grabbed_object);
+
+      XrdGrabEvent *event = g_malloc (sizeof (XrdGrabEvent));
+      event->controller = controller;
+      if (!gxr_controller_get_drag_pose (controller, &event->pose))
+        {
+          g_printerr ("Error while dragging window\n");
+          g_free (event);
+          return;
+        }
+
+      xrd_window_set_transformation (window, &event->pose);
+
+
+      xrd_window_emit_grab (window, event);
+    }
   else
-    _test_hover (self, pose, controller);
+    {
+      GxrHoverState *hover_state = gxr_controller_get_hover_state (controller);
+      XrdWindow *last_hover_target = XRD_WINDOW (hover_state->hovered_object);
+
+      graphene_point3d_t intersection_point;
+      float intersection_distance;
+      XrdWindow *hover_target = _get_hover_target (self,
+                                                   controller,
+                                                   &intersection_point,
+                                                   &intersection_distance);
+
+      graphene_point_t intersection_2d;
+      graphene_matrix_t hover_target_pose;
+      if (hover_target != NULL)
+        {
+          xrd_window_get_intersection_2d (hover_target,
+                                          &intersection_point,
+                                          &intersection_2d);
+
+          xrd_window_get_transformation_no_scale (hover_target,
+                                                  &hover_target_pose);
+        }
+
+
+      gxr_controller_update_hovered_object (controller,
+                                            last_hover_target,
+                                            hover_target,
+                                            &hover_target_pose,
+                                            &intersection_point,
+                                            &intersection_2d,
+                                            intersection_distance);
+
+
+      _emit_hover (self, controller, last_hover_target, hover_target,
+                   &intersection_point, intersection_distance);
+    }
+
 }
 
 GSList *
