@@ -9,6 +9,7 @@
 #include <gdk/gdk.h>
 
 #include "graphene-ext.h"
+#include "xrd-client-private.h"
 
 #define SCALE_MIN_FACTOR .05f
 #define SCALE_MAX_FACTOR 15.f
@@ -207,6 +208,45 @@ xrd_window_default_init (XrdWindowInterface *iface)
   iface->windows_created = 0;
 }
 
+XrdWindow *
+xrd_window_new_from_data (XrdClient     *self,
+                          XrdWindowData *data)
+{
+  return xrd_client_window_new_from_data (self, data);
+}
+
+XrdWindow *
+xrd_window_new_from_meters (XrdClient  *client,
+                            const char *title,
+                            float       width,
+                            float       height,
+                            float       ppm)
+{
+  return xrd_client_window_new_from_meters (client, title, width, height, ppm);
+}
+
+XrdWindow *
+xrd_window_new_from_pixels (XrdClient  *client,
+                            const char *title,
+                            uint32_t    width,
+                            uint32_t    height,
+                            float       ppm)
+{
+  return xrd_client_window_new_from_pixels (client, title, width, height, ppm);
+}
+
+XrdWindow *
+xrd_window_new_from_native (XrdClient   *client,
+                            const gchar *title,
+                            gpointer     native,
+                            uint32_t     width_pixels,
+                            uint32_t     height_pixels,
+                            float        ppm)
+{
+  return xrd_client_window_new_from_native (client, title, native,
+                                            width_pixels, height_pixels, ppm);
+}
+
 gboolean
 xrd_window_set_transformation (XrdWindow *self, graphene_matrix_t *mat)
 {
@@ -232,21 +272,19 @@ xrd_window_get_transformation_no_scale (XrdWindow         *self,
 /**
  * xrd_window_submit_texture:
  * @self: The #XrdWindow
- * @client: A #GulkanClient
  * After new content has been rendered into a texture, this function must be
  * called to reflect the changed texture on screen.
  */
 void
-xrd_window_submit_texture (XrdWindow *self, GulkanClient *client)
+xrd_window_submit_texture (XrdWindow *self)
 {
   XrdWindowInterface* iface = XRD_WINDOW_GET_IFACE (self);
-  iface->submit_texture (self, client);
+  iface->submit_texture (self);
 }
 
 /**
  * xrd_window_set_and_submit_texture:
  * @self: The #XrdWindow
- * @client: A #GulkanClient
  * @texture: A #GulkanTexture that is created by the caller.
  * Ownership of this texture is transferred to the #XrdWindow.
  *
@@ -261,11 +299,10 @@ xrd_window_submit_texture (XrdWindow *self, GulkanClient *client)
  */
 void
 xrd_window_set_and_submit_texture (XrdWindow     *self,
-                                   GulkanClient  *client,
                                    GulkanTexture *texture)
 {
   XrdWindowInterface* iface = XRD_WINDOW_GET_IFACE (self);
-  iface->set_and_submit_texture (self, client, texture);
+  iface->set_and_submit_texture (self, texture);
 }
 
 /**
@@ -364,33 +401,6 @@ xrd_window_poll_event (XrdWindow *self)
   iface->poll_event (self);
 }
 
-/**
- * xrd_window_intersects:
- * @self: The #XrdWindow
- * @pointer: The #XrdPointer.
- * @pointer_transformation: pose of a pointer ray (like a controller).
- * @intersection: The intersection point if there is an intersection.
- *
- * Returns: True if there is an intersection, else false.
- */
-gboolean
-xrd_window_intersects (XrdWindow          *self,
-                       XrdPointer         *pointer,
-                       graphene_matrix_t  *mat,
-                       graphene_point3d_t *intersection)
-{
-  (void) mat;
-
-  float distance;
-  graphene_vec3_t intersection_vec;
-  bool intersects = xrd_pointer_get_intersection (pointer, self,
-                                                  &distance, &intersection_vec);
-
-  graphene_point3d_init_from_vec3 (intersection, &intersection_vec);
-
-  return intersects;
-}
-
 /**
  * xrd_window_get_intersection_2d_pixels:
  * @self: The #XrdWindow
@@ -435,14 +445,16 @@ xrd_window_get_intersection_2d_pixels (XrdWindow          *self,
                       1.0f - graphene_vec2_get_y (&intersection_2d_vec));
 
   /* scale to pixel coordinates */
-  XrdPixelSize size_pixels;
+  VkExtent2D size_pixels;
   g_object_get (self,
                 "texture-width", &size_pixels.width,
                 "texture-height", &size_pixels.height,
                 NULL);
 
   graphene_vec2_t size_pixels_vec;
-  graphene_vec2_init (&size_pixels_vec, size_pixels.width, size_pixels.height);
+  graphene_vec2_init (&size_pixels_vec,
+                      (float) size_pixels.width,
+                      (float) size_pixels.height);
 
   graphene_vec2_multiply (&intersection_2d_vec, &size_pixels_vec,
                           &intersection_2d_vec);
@@ -482,10 +494,9 @@ xrd_window_get_intersection_2d (XrdWindow          *self,
 }
 
 void
-xrd_window_emit_grab_start (XrdWindow *self,
-                            XrdControllerIndexEvent *event)
+xrd_window_emit_grab_start (XrdWindow *self, GxrController *controller)
 {
-  g_signal_emit (self, window_signals[GRAB_START_EVENT], 0, event);
+  g_signal_emit (self, window_signals[GRAB_START_EVENT], 0, controller);
 }
 
 
@@ -497,17 +508,15 @@ xrd_window_emit_grab (XrdWindow *self,
 }
 
 void
-xrd_window_emit_release (XrdWindow *self,
-                         XrdControllerIndexEvent *event)
+xrd_window_emit_release (XrdWindow *self,GxrController *controller)
 {
-  g_signal_emit (self, window_signals[RELEASE_EVENT], 0, event);
+  g_signal_emit (self, window_signals[RELEASE_EVENT], 0, controller);
 }
 
 void
-xrd_window_emit_hover_end (XrdWindow *self,
-                           XrdControllerIndexEvent *event)
+xrd_window_emit_hover_end (XrdWindow *self, GxrController *controller)
 {
-  g_signal_emit (self, window_signals[HOVER_END_EVENT], 0, event);
+  g_signal_emit (self, window_signals[HOVER_END_EVENT], 0, controller);
 }
 
 
@@ -519,10 +528,9 @@ xrd_window_emit_hover (XrdWindow    *self,
 }
 
 void
-xrd_window_emit_hover_start (XrdWindow *self,
-                             XrdControllerIndexEvent *event)
+xrd_window_emit_hover_start (XrdWindow *self, GxrController *controller)
 {
-  g_signal_emit (self, window_signals[HOVER_START_EVENT], 0, event);
+  g_signal_emit (self, window_signals[HOVER_START_EVENT], 0, controller);
 }
 
 static void
@@ -753,10 +761,10 @@ xrd_window_get_reset_transformation (XrdWindow *self,
  * xrd_window_set_pin:
  * @self: The #XrdWindow
  * @pinned: The pin status to set this window to
- * @hide_unpinned: If true, the window will be hidden if it is unpinned, and
+ * @hide_unpinned: If TRUE, the window will be hidden if it is unpinned, and
  * shown if it is pinned. This corresponds to the "show only pinned windows"
  * mode set up in #XrdClient.
- * If false, windows are always shown.
+ * If FALSE, windows are always shown.
  * Note that @hide_unpinned only determines initial visibility, and does not
  * keep track of further mode changes.
  */
@@ -804,8 +812,4 @@ xrd_window_close (XrdWindow *self)
    * but sometimes it can happen. */
   if (data->child_window)
     data->child_window->parent_window = NULL;
-
-
-  if (data->title)
-    g_string_free (data->title, TRUE);
 }
